//
//  RootViewController.swift
//  UIKitExample
//
//  Created by Yutaro Muta on 2020/11/30.
//

import UIKit

class RootViewController: UIViewController {
    private enum Section: Int {
        case picker = 0
        case collection

        var items: [ViewController] {
            switch self {
            case .picker:
                return [.datePicker, .colorPicker, .menu]
            case .collection:
                return [.basic, .diffable, .compositional, .list]
            }
        }
    }

    private enum ViewController: String, CaseIterable {
        case datePicker = "DatePickerViewController"
        case colorPicker = "ColorPickerViewController"
        case menu = "MenuViewController"
        case basic = "BasicCollectionViewController"
        case diffable = "DiffableCollectionViewController"
        case compositional = "CompositionalViewController"
        case list = "ListCollectionViewController"

        var instance: UIViewController {
            switch self {
            case .datePicker:
                return DatePickerViewController()
            case .colorPicker:
                return ColorPickerViewController()
            case .menu:
                return MenuViewController()
            case .basic, .diffable, .compositional, .list:
                let storyboard = UIStoryboard(name: "Main", bundle: nil)
                return storyboard.instantiateViewController(identifier: rawValue)
            }
        }
    }

    @IBOutlet private weak var collectionView: UICollectionView! {
        didSet {
            collectionView.delegate = self
            let configuration = UICollectionLayoutListConfiguration(appearance: .grouped)
            collectionView.collectionViewLayout = UICollectionViewCompositionalLayout.list(using: configuration)
            var snapshot = NSDiffableDataSourceSnapshot<Section, ViewController>()
            snapshot.appendSections([.picker, .collection])
            snapshot.appendItems(Section.picker.items, toSection: .picker)
            snapshot.appendItems(Section.collection.items, toSection: .collection)
            dataSource.apply(snapshot)
        }
    }

    private lazy var dataSource: UICollectionViewDiffableDataSource<Section, ViewController> = {
        let cellRegistration: UICollectionView.CellRegistration<UICollectionViewListCell, ViewController> =
            .init { cell, indexPath, viewController in
                var contentConfiguration = cell.defaultContentConfiguration()
                contentConfiguration.text = viewController.rawValue
                cell.contentConfiguration = contentConfiguration
                cell.accessories = [.disclosureIndicator()]
            }
        return .init(collectionView: self.collectionView,
                     cellProvider: { collectionView, indexPath, device in
                        return collectionView.dequeueConfiguredReusableCell(using: cellRegistration,
                                                                            for: indexPath,
                                                                            item: device)
                     }
        )
    }()
}

extension RootViewController: UICollectionViewDelegate {
    func collectionView(_ collectionView: UICollectionView, didSelectItemAt indexPath: IndexPath) {
        guard let section = Section(rawValue: indexPath.section) else {
            return
        }
        let viewController = section.items[indexPath.row].instance
        navigationController?.pushViewController(viewController, animated: true)
    }
}
