//
//  ViewController.swift
//  Scribble
//
//  Created by Kouki Saito on 2020/12/20.
//

import UIKit

class ViewController: UIViewController {
    let customLabelInput = CustomLabelInput()

    override func viewDidLoad() {
        super.viewDidLoad()
        view.addSubview(customLabelInput)
        customLabelInput.translatesAutoresizingMaskIntoConstraints = false
        customLabelInput.centerXAnchor.constraint(equalTo: view.centerXAnchor).isActive = true
        customLabelInput.centerYAnchor.constraint(equalTo: view.topAnchor, constant: 120).isActive = true
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(handleTapGesture))
        view.addGestureRecognizer(tapGesture)
    }

    @objc func handleTapGesture() {
        customLabelInput.finishEditing()
    }
}

class CustomLabelInput: UIView {
    var label = UILabel()
    var editingTextField: UITextField?
    var indirectScribbleInteraction: UIIndirectScribbleInteraction<CustomLabelInput>!
    var text = "Scribble Me!"
    var isEditing = false {
        didSet {
            if oldValue != isEditing {
                updateView()
            }
        }
    }

    override var intrinsicContentSize: CGSize {
        return CGSize(width: 300, height: 30)
    }

    // MARK: - UIView Overrides
    override init(frame: CGRect) {
        super.init(frame: frame)

        indirectScribbleInteraction = UIIndirectScribbleInteraction(delegate: self)
        addInteraction(indirectScribbleInteraction)

        updateView()
        let tapGesture = UITapGestureRecognizer(target: self, action: #selector(handleTapGesture))
        addGestureRecognizer(tapGesture)
    }

    required init?(coder: NSCoder) {
        fatalError("Not implemented")
    }

    // MARK: - UI Elements Updating
    func updateView() {
        backgroundColor = .systemGray5
        if isEditing && editingTextField == nil {
            let newTextField = UITextField(frame: bounds)
            newTextField.addTarget(self, action: #selector(handleTextFieldDidChange(_:)), for: .editingChanged)
            newTextField.text = text
            newTextField.textAlignment = .center
            addSubview(newTextField)
            editingTextField = newTextField
        } else if !isEditing, let textFieldToRemove = editingTextField {
            textFieldToRemove.resignFirstResponder()
            textFieldToRemove.removeFromSuperview()
            editingTextField = nil
        }
        if label.superview == nil {
            label.textAlignment = .center
            addSubview(label)
            label.translatesAutoresizingMaskIntoConstraints = false
            NSLayoutConstraint.activate([
                label.leadingAnchor.constraint(equalTo: leadingAnchor),
                label.topAnchor.constraint(equalTo: topAnchor),
                label.trailingAnchor.constraint(equalTo: trailingAnchor),
                label.bottomAnchor.constraint(equalTo: bottomAnchor),
            ])
        }
        label.text = text
        label.isHidden = isEditing
    }

    func finishEditing() {
        isEditing = false
    }

    // MARK: - Event Handling
    @objc
    func handleTextFieldDidChange(_ textField: UITextField) {
        // Keep track of the current engraving string.
        text = textField.text ?? ""
    }

    @objc
    func handleTapGesture() {
        // On tap, install the text field and focus it.
        isEditing = true
        editingTextField!.becomeFirstResponder()
    }
}

extension CustomLabelInput: UIIndirectScribbleInteractionDelegate {
    func indirectScribbleInteraction(_ interaction: UIInteraction, requestElementsIn rect: CGRect,
                                     completion: @escaping ([ElementIdentifier]) -> Void) {
        completion(["ElementIdentifier"])
    }

    func indirectScribbleInteraction(_ interaction: UIInteraction, frameForElement elementIdentifier: String) -> CGRect {
        return bounds
    }

    func indirectScribbleInteraction(_ interaction: UIInteraction, isElementFocused elementIdentifier: String) -> Bool {
        return editingTextField?.isFirstResponder ?? false
    }

    func indirectScribbleInteraction(_ interaction: UIInteraction, focusElementIfNeeded elementIdentifier: String,
                                     referencePoint focusReferencePoint: CGPoint, completion: @escaping ((UIResponder & UITextInput)?) -> Void) {
        isEditing = true
        completion(editingTextField)
    }

    func indirectScribbleInteraction(_ interaction: UIInteraction, willBeginWritingInElement elementIdentifier: String) {
        updateView()
    }

    func indirectScribbleInteraction(_ interaction: UIInteraction, didFinishWritingInElement elementIdentifier: String) {
        updateView()
    }
}
